
/******************************************************************************
* MODULE     : analytic_test.cc
* DESCRIPTION: Test analytic functions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "analytic.hpp"
using namespace mml;

//#define Complex    complexify<double>
//#define Polynomial polynomial<Complex>
//#define Analytic   analytic<double_analytic>

#define Real       real
#define Complex    complex
#define Polynomial polynomial<Complex>
#define Analytic   analytic<exact_analytic>

int
main () {
  //Complex I (Real (0), Real (1));
  Complex I (0, 1);
  Polynomial Z= seq (Complex (0), Complex (1));
  Polynomial P= seq (Complex (1), Complex (2), Complex (3));
  Analytic f (1);
  Analytic g (P);
  Analytic z (Z);
  Analytic h= integrate (exp (-z * z));
  cout << "f\t\t= " << f << "\n";
  cout << "g\t\t= " << g << "\n";
  //cout << "h\t\t= " << h << "\n";
  cout << "f->i\t\t= " << f->move (I) << "\n";
  cout << "g->i\t\t= " << g->move (I) << "\n";
  cout << "f+g\t\t= " << f+g << "\n";
  cout << "f-g\t\t= " << f-g << "\n";
  cout << "-g\t\t= " << -g << "\n";
  cout << "2 g\t\t= " << Complex (2) * g << "\n";
  cout << "g/2\t\t= " << g / Complex (2) << "\n";
  cout << "g g\t\t= " << g*g << "\n";
  cout << "f/g\t\t= " << f/g << "\n";
  cout << "r(f)\t\t= " << f -> radius () << "\n";
  cout << "r(g)\t\t= " << g -> radius () << "\n";
  cout << "r(f/g)\t\t= " << (f/g) -> radius () << "\n";
  cout << "(f/g)->i\t= " << (f/g) -> continuate (I) << "\n";
  cout << "exp(z)\t\t= " << exp (z) << "\n";
  cout << "exp(g)\t\t= " << exp (g) << "\n";
  cout << "g'\t\t= " << derive (g) << "\n";
  cout << "h\t\t= " << h << "\n";
  cout << "r(h)\t\t= " << h -> radius () << "\n";
  cout << "h(1)\t\t= " << h->evaluate (1) << "\n";
  cout << "h->1\t\t= " << h->continuate (1) << "\n";
  cout << "h->2\t\t= " << h->continuate (2) << "\n";
  cout << "h->1->1\t\t= " << h->continuate (1)->continuate (1) << "\n";
  cout << "h->5\t\t= " << h->continuate (5) << "\n";
  // cout << "h->10\t\t= " << h->continuate (10) << "\n";
  cout << "log(1+z)\t= " << log (f+z) << "\n";
  cout << "log(1+z)->turn\t= " <<
    log (f+z)->continuate (I-2)->continuate (-2*I)->continuate (I+2) << "\n";
  cout << "exp(z)\t\t= " << solve_lde (vec (f), vec (Complex (1))) << "\n";
  cout << "exp(1)\t\t= " <<
    solve_lde (vec (f), vec (Complex (1)))->evaluate (1) << "\n";
  Analytic ll= solve_lde (vec (Analytic (0), -f/(f+z)),
			  vec (Complex (0), Complex (1)));
  cout << "log(1+z)\t= " << ll << "\n";
  cout << "r(log(1+z))\t= " << ll->radius () << "\n";
  cout << "log(21/20)\t= " <<
    ll->evaluate (Complex (1) / Complex (20)) << "\n";
  cout << "log(1+z)->21/20\t= " <<
    ll->continuate (Complex (1) / Complex (20)) << "\n";
  cout << "log(1+z)->turn\t= " <<
    ll->continuate (I-2)->continuate (-2*I)->continuate (I+2) << "\n";
  return 0;
}
