
/******************************************************************************
* MODULE     : wrap.hpp
* DESCRIPTION: Macros for the construction of wrapper classes
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_WRAP_HPP
#define __MML_WRAP_HPP
namespace mml {

/******************************************************************************
* Basic operations
******************************************************************************/

#define WRAP_BASIC_PROTO(TMPL,C)			\
TMPL nat hash (const C& c);				\
TMPL generic as_generic (const C& c);			\
TMPL ostream& operator << (ostream& out, const C& c);

#define WRAP_BASIC_IMPL(TMPL,C)						      \
TMPL nat hash (const C& c) { return hash (c.rep); }			      \
TMPL generic as_generic (const C& c) { return as_generic (c.rep); }	      \
TMPL ostream& operator << (ostream& out, const C& c) { return out << c.rep; }

/******************************************************************************
* Field operations
******************************************************************************/

#define WRAP_FIELD_PROTO(TMPL,C)		\
TMPL C operator - (const C& c);			\
TMPL C operator + (const C& c1, const C& c2);	\
TMPL C operator - (const C& c1, const C& c2);	\
TMPL C operator * (const C& c1, const C& c2);	\
TMPL C operator / (const C& c1, const C& c2);	\
TMPL C& operator += (C& c1, const C& c2);	\
TMPL C& operator -= (C& c1, const C& c2);	\
TMPL C& operator *= (C& c1, const C& c2);	\
TMPL C& operator /= (C& c1, const C& c2);

#define WRAP_FIELD_IMPL(TMPL,C)							\
TMPL C operator - (const C& c) { return C(-c.rep); }				\
TMPL C operator + (const C& c1, const C& c2) { return C (c1.rep + c2.rep); }	\
TMPL C operator - (const C& c1, const C& c2) { return C (c1.rep - c2.rep); }	\
TMPL C operator * (const C& c1, const C& c2) { return C (c1.rep * c2.rep); }	\
TMPL C operator / (const C& c1, const C& c2) { return C (c1.rep / c2.rep); }	\
TMPL C& operator += (C& c1, const C& c2) { c1.rep += c2.rep; return c1; }	\
TMPL C& operator -= (C& c1, const C& c2) { c1.rep -= c2.rep; return c1; }	\
TMPL C& operator *= (C& c1, const C& c2) { c1.rep *= c2.rep; return c1; }	\
TMPL C& operator /= (C& c1, const C& c2) { c1.rep /= c2.rep; return c1; }

#define WRAP_SHIFT_PROTO(TMPL,C)		\
TMPL C operator << (const C& c, int sh);	\
TMPL C operator >> (const C& c, int sh);	\
TMPL C& operator <<= (C& c, int sh);		\
TMPL C& operator >>= (C& c, int sh);

#define WRAP_SHIFT_IMPL(TMPL,C)						\
TMPL C operator << (const C& c, int sh) { return C (c.rep << sh); }	\
TMPL C operator >> (const C& c, int sh) { return C (c.rep >> sh); }	\
TMPL C& operator <<= (C& c, int sh) { c.rep << sh; return c; }		\
TMPL C& operator >>= (C& c, int sh) { c.rep >> sh; return c; }

/******************************************************************************
* Ordering
******************************************************************************/

#define WRAP_EQUAL_PROTO(TMPL,C)			\
TMPL bool operator == (const C& c1, const C& c2);	\
TMPL bool operator != (const C& c1, const C& c2);

#define WRAP_EQUAL_IMPL(TMPL,C)						      \
TMPL bool operator == (const C& c1, const C& c2) { return c1.rep == c2.rep; } \
TMPL bool operator != (const C& c1, const C& c2) { return c1.rep != c2.rep; }

#define WRAP_COMPARE_PROTO(TMPL,C)			\
WRAP_EQUAL_PROTO(TMPL,C);				\
TMPL bool operator <= (const C& c1, const C& c2);	\
TMPL bool operator <  (const C& c1, const C& c2);	\
TMPL bool operator >= (const C& c1, const C& c2);	\
TMPL bool operator >  (const C& c1, const C& c2);

#define WRAP_COMPARE_IMPL(TMPL,C)					      \
WRAP_EQUAL_IMPL(TMPL,C);						      \
TMPL bool operator <= (const C& c1, const C& c2) { return c1.rep <= c2.rep; } \
TMPL bool operator <  (const C& c1, const C& c2) { return c1.rep <  c2.rep; } \
TMPL bool operator >= (const C& c1, const C& c2) { return c1.rep >= c2.rep; } \
TMPL bool operator >  (const C& c1, const C& c2) { return c1.rep >  c2.rep; }

#define WRAP_RING_EQUAL_PROTO(TMPL,C)		\
WRAP_EQUAL_PROTO(TMPL,C);			\
TMPL bool is_zero (const C& c);			\
TMPL bool is_non_zero (const C& c);

#define WRAP_RING_EQUAL_IMPL(TMPL,C)					\
WRAP_EQUAL_IMPL(TMPL,C);						\
TMPL bool is_zero (const C& c) { return is_zero (c.rep); }		\
TMPL bool is_non_zero (const C& c) { return is_non_zero (c.rep); }

#define WRAP_RING_COMPARE_PROTO(TMPL,C)		\
WRAP_COMPARE_PROTO(TMPL,C);			\
TMPL bool is_zero (const C& c);			\
TMPL bool is_non_zero (const C& c);		\
TMPL int  sign (const C& c);			\
TMPL C abs (const C& c);			\
TMPL C min (const C& c1, const C& c2);		\
TMPL C max (const C& c1, const C& c2);

#define WRAP_RING_COMPARE_IMPL(TMPL,C)					   \
WRAP_COMPARE_IMPL(TMPL,C);						   \
TMPL bool is_zero (const C& c) { return is_zero (c.rep); }		   \
TMPL bool is_non_zero (const C& c) { return is_non_zero (c.rep); }	   \
TMPL int  sign (const C& c) { return sign (c.rep); }			   \
TMPL C abs (const C& c) { return C (abs (c.rep)); }			   \
TMPL C min (const C& c1, const C& c2) { return C (min (c1.rep, c2.rep)); } \
TMPL C max (const C& c1, const C& c2) { return C (max (c1.rep, c2.rep)); }

/******************************************************************************
* Checks for numbers
******************************************************************************/

#define WRAP_NUMBER_CHECK_PROTO(TMPL,C)		\
TMPL bool is_number (const C& c);		\
TMPL bool is_infinity (const C& c);		\
TMPL bool is_fuzz (const C& c);			\
TMPL bool is_nan (const C& c);

#define WRAP_NUMBER_CHECK_IMPL(TMPL,C)					\
TMPL bool is_number (const C& c) { return is_number (c.rep); }		\
TMPL bool is_infinity (const C& c) { return is_infinity (c.rep); }	\
TMPL bool is_fuzz (const C& c) { return is_nan (c.rep); }		\
TMPL bool is_nan (const C& c) { return is_nan (c.rep); }

/******************************************************************************
* Special functions
******************************************************************************/

#define WRAP_SPECIAL_PROTO(TMPL,C)		\
TMPL C sqrt (const C& c);			\
TMPL C exp (const C& c);			\
TMPL C log (const C& c);			\
TMPL C pow (const C& c1, const C& c2);		\
TMPL C cos (const C& c);			\
TMPL C sin (const C& c);			\
TMPL C tan (const C& c);			\
TMPL C acos (const C& c);			\
TMPL C asin (const C& c);			\
TMPL C atan (const C& c);			\
TMPL C cosh (const C& c);			\
TMPL C sinh (const C& c);			\
TMPL C tanh (const C& c);			\
TMPL C acosh (const C& c);			\
TMPL C asinh (const C& c);			\
TMPL C atanh (const C& c);

#define WRAP_SPECIAL_IMPL(TMPL,C)					   \
TMPL C sqrt (const C& c) { return C (sqrt (c.rep)); }			   \
TMPL C exp (const C& c) { return C (exp (c.rep)); }			   \
TMPL C log (const C& c) { return C (log (c.rep)); }			   \
TMPL C pow (const C& c1, const C& c2) { return C (pow (c1.rep, c2.rep)); } \
TMPL C cos (const C& c) { return C (cos (c.rep)); }			   \
TMPL C sin (const C& c) { return C (sin (c.rep)); }			   \
TMPL C tan (const C& c) { return C (tan (c.rep)); }			   \
TMPL C acos (const C& c) { return C (acos (c.rep)); }			   \
TMPL C asin (const C& c) { return C (asin (c.rep)); }			   \
TMPL C atan (const C& c) { return C (atan (c.rep)); }			   \
TMPL C cosh (const C& c) { return C (cosh (c.rep)); }			   \
TMPL C sinh (const C& c) { return C (sinh (c.rep)); }			   \
TMPL C tanh (const C& c) { return C (tanh (c.rep)); }			   \
TMPL C acosh (const C& c) { return C (acosh (c.rep)); }			   \
TMPL C asinh (const C& c) { return C (asinh (c.rep)); }			   \
TMPL C atanh (const C& c) { return C (atanh (c.rep)); }

/******************************************************************************
* Extra routines for real-like and complex-like types
******************************************************************************/

#define WRAP_REAL_PROTO(TMPL,C)			\
TMPL C hypot (const C& c1, const C& c2);	\
TMPL C atan2 (const C& c1, const C& c2);

#define WRAP_REAL_IMPL(TMPL,C)			\
TMPL C hypot (const C& c1, const C& c2) {	\
  return C (hypot (c1.rep, c2.rep)); }		\
TMPL C atan2 (const C& c1, const C& c2) {	\
  return C (atan2 (c1.rep, c2.rep)); }

#define WRAP_COMPLEX_PROTO(TMPL,C,R)		\
TMPL C gaussian (const R& c1, const R& c2= 0);	\
TMPL C polar (const R& c1, const R& c2);	\
TMPL R Re (const C& c);				\
TMPL R Im (const C& c);				\
TMPL R abs (const C& c);			\
TMPL R arg (const C& c);			\
TMPL R conj (const C& c);

#define WRAP_COMPLEX_IMPL(TMPL,C,R)			\
TMPL C gaussian (const R& c1, const R& c2) {		\
  return C (gaussian (c1.rep, c2.rep)); }		\
TMPL C polar (const R& c1, const R& c2) {		\
  return C (polar (c1.rep, c2.rep)); }			\
TMPL R Re (const C& c) { return R (Re (c.rep)); }	\
TMPL R Im (const C& c) { return R (Im (c.rep)); }	\
TMPL R abs (const C& c) { return R (abs (c.rep)); }	\
TMPL R arg (const C& c) { return R (arg (c.rep)); }	\
TMPL C conj (const C& c) { return C (conj (c.rep)); }

} // namespace mml
#endif // __MML_WRAP_HPP
