
/******************************************************************************
* MODULE     : function.hpp
* DESCRIPTION: Functions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_FUNCTION_HPP
#define __MML_FUNCTION_HPP
#include "string.hpp"
#include "table.hpp"
namespace mml {
#define TMPL_DEF template<typename C, typename T, typename V=plain_function>
#define TMPL template<typename C, typename T, typename V>
#define Function function<C,T,V>
#define Function_rep function_rep<C,T,V>
TMPL class function_rep;
TMPL class function;
TMPL inline C apply_function (const Function_rep* rep, const T& x);

/******************************************************************************
* Functions
******************************************************************************/

class plain_function {};
TMPL class function_cache {
public:
  inline function_cache () {}
  static inline C apply (Function_rep* rep, const T& x);
};

TMPL_DEF
class function_rep REP_STRUCT {
protected:
  function_cache<C,T,V> cache;

public:
  inline function_rep () {}
  virtual ~function_rep () {}
  virtual generic expression () const = 0;
  virtual C eval (const T& x) = 0;
  friend class function_cache<C,T,V>;
  friend class Function;
};

TMPL inline C
function_cache<C,T,V>::apply (Function_rep* rep, const T& x) {
  return rep->eval (x);
}

TMPL_DEF
class function {
INDIRECT_PROTO_3 (function, function_rep, C, T, V)
  function ();
  function (int i);
  function (const C& init);
  function (const table<C,T>& t);
  function (C (*fun) (T), string name= "explicit");
  inline C operator () (const T& x) {
    return function_cache<C,T,V>::apply (rep, x); }
};
INDIRECT_IMPL_3 (function, function_rep,
		 typename C, C, typename T, T, typename V, V)

TMPL generic as_generic (const Function& f) { return f->expression (); }
PRINT_SUGAR(TMPL,Function)

/******************************************************************************
* Functions with cache
******************************************************************************/

class remember_function {};
template<typename C,typename T>
class function_cache<C,T,remember_function> {
public:
  table<C,T> t; // remembered values
  inline function_cache (): t() {}
  static inline C apply (function_rep<C,T,remember_function>* rep, const T& x);
};

template<typename C,typename T> inline C
function_cache<C,T,remember_function>::
apply (function_rep<C,T,remember_function>* rep, const T& x) {
  if (!rep->cache.t->contains (x)) {
    C val= rep->eval (x);
    rep->cache.t (x)= val;
  }
  return rep->cache.t[x];
}

/******************************************************************************
* Constant functions
******************************************************************************/

TMPL_DEF
class constant_function_rep: public Function_rep {
  C c;
public:
  constant_function_rep (const C& c2): c (c2) {}
  generic expression () const { return as_generic (c); }
  C eval (const T& t) { return c; }
};

TMPL Function::function () {
  rep= new constant_function_rep<C,T,V> (C ()); }
TMPL Function::function (int c) {
  rep= new constant_function_rep<C,T,V> (C (c)); }
TMPL Function::function (const C& c) {
  rep= new constant_function_rep<C,T,V> (c); }

/******************************************************************************
* Explicit functions
******************************************************************************/

TMPL_DEF
class table_function_rep: public Function_rep {
  table<C,T> u;
public:
  table_function_rep (const table<C,T>& u2): u (u2) {}
  generic expression () const { return as_generic (u); }
  C eval (const T& x) { return u[x]; }
};

TMPL Function::function (const table<C,T>& t) {
  rep= new table_function_rep<C,T,V> (t); }

TMPL_DEF
class cpp_function_rep: public Function_rep {
  C (*fun) (T);
  string name;
public:
  cpp_function_rep (C (*fun2) (T), string name2): fun (fun2), name (name2) {}
  generic expression () const { return as_generic (name); }
  C eval (const T& x) { return fun (x); }
};

TMPL Function::function (C (*fun) (T), string name) {
  rep= new cpp_function_rep<C,T,V> (fun, name); }

/******************************************************************************
* Vectorial arithmetic on functions
******************************************************************************/

#define FUNCTION_BINARY(binop_function_rep,op,disp)			\
TMPL_DEF								\
class binop_function_rep: public Function_rep {				\
  Function f, g;							\
public:									\
  binop_function_rep (const Function& f2, const Function& g2):		\
    f (f2), g (g2) {}							\
  generic expression () const { return as_generic (disp, f, g); }	\
  C eval (const T& x) { return f(x) op g(x); }				\
};									\
									\
TMPL Function								\
operator op (const Function& f, const Function& g) {			\
  return Function (new binop_function_rep<C,T,V> (f, g));		\
}

FUNCTION_BINARY(plus_function_rep,+,"+");
FUNCTION_BINARY(minus_function_rep,-,"-");
FUNCTION_BINARY(times_function_rep,*,"*");
FUNCTION_BINARY(over_function_rep,/,"/");
ARITH_INT_SUGAR(TMPL,Function,C);
ARITH_SCALAR_SUGAR(TMPL,Function,C);

#undef TMPL_DEF
#undef TMPL
#undef Function
#undef Function_rep
} // namespace mml
#endif // __MML_FUNCTION_HPP
