
/******************************************************************************
* MODULE     : floating_ball.hpp
* DESCRIPTION: Converters for floating point balls
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_FLOATING_BALL_HPP
#define __MML_FLOATING_BALL_HPP
#include "rounded_floating.hpp"
#include "complexify.hpp"
#include "ball.hpp"
namespace mml {

template<typename W,typename V>
class ball_rounding_helper<floating<W>,floating<V> > {
public:
  static inline floating<W> cv_D (const floating<V>& x) {
    floating<W> r; mpfr_set (*r, *x, GMP_RNDD); return r; }
  static inline floating<W> cv_U (const floating<V>& x) {
    floating<W> r; mpfr_set (*r, *x, GMP_RNDU); return r; }
  static inline floating<W> cv_abs_D (const floating<V>& x) {
    return cv_D (abs (x)); }
  static inline floating<W> cv_abs_U (const floating<V>& x) {
    return cv_U (abs (x)); }
};

template<typename W,typename V>
class ball_rounding_helper<floating<W>,complexify<floating<V> > > {
public:
  static inline floating<W> cv_D (const floating<V>& x) {
    floating<W> r; mpfr_set (*r, *x, GMP_RNDD); return r; }
  static inline floating<W> cv_U (const floating<V>& x) {
    floating<W> r; mpfr_set (*r, *x, GMP_RNDU); return r; }
  static inline floating<W> cv_abs_D (const complexify<floating<V> >& z) {
    return cv_D (hypot_D (Re (z), Im (z))); }
  static inline floating<W> cv_abs_U (const complexify<floating<V> >& z) {
    return cv_U (hypot_U (Re (z), Im (z))); }
};

} // namespace mml
#endif // __MML_FLOATING_BALL_HPP
