
/******************************************************************************
* MODULE     : interval_test.cc
* DESCRIPTION: Test intervals
* COPYRIGHT  : (C) 2006  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "rounded_floating.hpp"
#include "interval.hpp"
using namespace mml;

#define Float floating<>
#define Interval interval<Float >

void
test_special () {
  cout << "*** Special values\n\n";
  //cout << "epsilon\t= " << Epsilon (Interval) << "\n";
  cout << "fuzzy\t= " << Fuzz (Interval) << "\n";
  cout << "\n";
}

void
test_add_simple (int prec) {
  set_precision (prec);
  cout << "*** Additive arithmetic at precision " << prec << "\n\n";
  Interval x= 2;
  Interval y= 7;
  cout << "x\t= " << x << "\n";
  cout << "y\t= " << y << "\n";
  cout << "-x\t= " << (-x) << "\n";
  cout << "x+x\t= " << (x+x) << "\n";
  cout << "x+y\t= " << (x+y) << "\n";
  cout << "x-x\t= " << (x-x) << "\n";
  cout << "x-y\t= " << (x-y) << "\n";
  cout << "(x-x)+y\t= " << ((x-x)+y) << "\n";
  cout << "\n";
}

void
test_mul_simple (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative arithmeric at precision " << prec << "\n\n";
  Interval x= Interval (2) + Interval (1);
  cout << "x\t= " << x << "\n";
  cout << "x * x\t= " << x*x << "\n";
  cout << "x * -x\t= " << x*(-x) << "\n";
  cout << "-x * x\t= " << (-x)*x << "\n";
  cout << "-x * -x\t= " << (-x)*(-x) << "\n";
  cout << "x / x\t= " << (x/x) << "\n";
  cout << "x / -x\t= " << (x/(-x)) << "\n";
  cout << "-x / x\t= " << ((-x)/x) << "\n";
  cout << "-x / -x\t= " << ((-x)/(-x)) << "\n";
  cout << "x*x / x\t= " << ((x*x)/x) << "\n";
  cout << "\n";
}

void
test_other_ops (int prec) {
  set_precision (prec);
  cout << "*** Other operations " << prec << "\n\n";
  Interval x= Interval (2);
  Interval y (-3, 2);
  cout << "x\t= " << x << "\n";
  cout << "y\t= " << y << "\n";
  cout << "x << 5\t= " << (x << 5) << "\n";
  cout << "x >> 3\t= " << (x >> 3) << "\n";
  cout << "y << 5\t= " << (y << 5) << "\n";
  cout << "abs(x)\t= " << (abs (x)) << "\n";
  cout << "abs(-x)\t= " << (abs (-x)) << "\n";
  cout << "abs(y)\t= " << (abs (y)) << "\n";
  cout << "\n";
}

void
test_mul_progression (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative progression at precision " << prec << "\n\n";
  int i;
  Interval x= 3;
  Interval y= x;
  for (i=0; i<50; i++) {
    cout << i << "\t" << y << "\n";
    y *= x;
  }
  cout << "\n";
}

void
test_mul_overflow (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative overflows at precision " << prec << "\n\n";
  int i;
  Interval x= 3;
  for (i=0; i<50; i++) {
    cout << i << "\t" << x << "\n";
    x *= x;
  }
  cout << "\n";
}

void
test_mul_underflow (int prec) {
  set_precision (prec);
  cout << "*** Multiplicative underflows at precision " << prec << "\n\n";
  int i;
  Interval x= 0.3;
  for (i=0; i<50; i++) {
    cout << i << "\t" << x << "\n";
    x *= x;
  }
  cout << "\n";
}

void
test_muller (int prec) {
  set_precision (prec);
  cout << "*** Muller's sequence at precision " << prec << "\n\n";
  int i;
  Interval a= 99;
  Interval b= -600;
  Interval d= -7;
  Interval x= 4;
  for (i=0; i<50; i++) {
    cout << i << "\t" << x << "\n";
    x= (a*x + b) / (x + d);
  }
  cout << "\n";
}

void
test_special_funcs (int prec) {
  set_precision (prec);
  cout << "*** Special functions at precision " << prec << "\n\n";
  Interval x= 1;
  Interval y= 3e20;
  Interval z= x - x/y;
  cout << "log (2)\t\t= " << Log2 (Interval) << "\n";
  cout << "pi\t\t= " << Pi (Interval) << "\n";
  cout << "euler\t\t= " << Euler (Interval) << "\n";
  cout << "NaN\t\t= " << Nan (Interval) << "\n";
  cout << "Infty\t\t= " << Plus_infinity (Interval) << "\n";
  cout << "Fuzz\t\t= " << Fuzz (Interval) << "\n";
  cout << "x\t\t= " << x << "\n";
  cout << "y\t\t= " << y << "\n";
  cout << "z\t\t= " << z << "\n";
  cout << "sqrt (x)\t= " << sqrt (x) << "\n";
  cout << "sqrt (y)\t= " << sqrt (y) << "\n";
  cout << "sqrt (-y)\t= " << sqrt (-y) << "\n";
  cout << "exp (x)\t\t= " << exp (x) << "\n";
  cout << "exp (y)\t\t= " << exp (y) << "\n";
  cout << "exp (-x)\t= " << exp (-x) << "\n";
  cout << "exp (-y)\t= " << exp (-y) << "\n";
  cout << "log (x)\t\t= " << log (x) << "\n";
  cout << "log (y)\t\t= " << log (y) << "\n";
  cout << "cos (x)\t\t= " << cos (x) << "\n";
  cout << "cos (y)\t\t= " << cos (y) << "\n";
  cout << "sin (x)\t\t= " << sin (x) << "\n";
  cout << "sin (y)\t\t= " << sin (y) << "\n";
  cout << "tan (x)\t\t= " << tan (x) << "\n";
  cout << "tan (y)\t\t= " << tan (y) << "\n";
  cout << "arc sin (1/2)\t= " << asin (x>>1) << "\n";
  cout << "arc sin (z)\t= " << asin (z) << "\n";
  cout << "arc cos (1/2)\t= " << acos (x>>1) << "\n";
  cout << "arc cos (z)\t= " << acos (z) << "\n";
  cout << "arc tan (x)\t= " << atan (x) << "\n";
  cout << "arc tan (y)\t= " << atan (y) << "\n";
  cout << "arc tan (x)\t= " << atan (x) << "\n";
  cout << "atan2 (x, y)\t= " << atan2 (x, y) << "\n";
  cout << "atan2 (y, x)\t= " << atan2 (y, x) << "\n";
  cout << "atan2 (0, -x)\t= " << atan2 (Interval (0), -x) << "\n";
  cout << "min (x, y)\t= " << min (x, y) << "\n";
  cout << "max (x, y)\t= " << max (x, y) << "\n";
  cout << "\n";
}

void
test_all (int prec) {
  test_add_simple (prec);
  test_mul_simple (prec);
  test_other_ops (prec);
  test_mul_progression (prec);
  test_mul_overflow (prec);
  test_mul_underflow (prec);
  test_muller (prec);
  test_special_funcs (prec);
}

int
main () {
  test_special ();
  test_all (30);
  test_all (100);
  test_all (200);

  cout << "infty\t\t= " << Plus_infinity (Interval) << "\n";
  cout << "-infty\t\t= " << Minus_infinity (Interval) << "\n";
  cout << "infty+1\t\t= " << Plus_infinity (Interval) + Interval (1) << "\n";
  cout << "1-infty\t\t= " << Interval (1) - Plus_infinity (Interval) << "\n";
  cout << "1+(-infty)\t= " << Interval (1) + Minus_infinity (Interval) << "\n";
  cout << "2 infty\t\t= " << Interval (2) * Plus_infinity (Interval) << "\n";
  cout << "1/infty\t\t= " << Interval (1) / Plus_infinity (Interval) << "\n";
  cout << "infty^2\t\t= " <<
    Plus_infinity (Interval) * Plus_infinity (Interval) << "\n";
  cout << "(-infty) infty\t= " <<
    Minus_infinity (Interval) * Plus_infinity (Interval) << "\n";
  cout << "(-infty)^2\t= " <<
    Minus_infinity (Interval) * Minus_infinity (Interval) << "\n";
  cout << "infty/infty\t= " <<
    Plus_infinity (Interval) / Plus_infinity (Interval) << "\n";
  cout << "(-infty)/infty\t= " <<
    Minus_infinity (Interval) / Plus_infinity (Interval) << "\n";
  cout << "(-infty)/2\t= " << Minus_infinity (Interval) / Interval (2) << "\n";
  cout << "|infty|\t\t= " << abs (Plus_infinity (Interval)) << "\n";
  cout << "|-infty|\t= " << abs (Minus_infinity (Interval)) << "\n";
  cout << "center(infty)\t= " << center (Plus_infinity (Interval)) << "\n";
  cout << "center(-infty)\t= " << center (Minus_infinity (Interval)) << "\n";
  cout << "radius(infty)\t= " << radius (Plus_infinity (Interval)) << "\n";
  cout << "radius(-infty)\t= " << radius (Minus_infinity (Interval)) << "\n";
  cout << "infty << 3\t= " << (Plus_infinity (Interval) << 3) << "\n";
  cout << "infty >> 3\t= " << (Plus_infinity (Interval) >> 3) << "\n";
  cout << "(-infty) << 3\t= " << (Minus_infinity (Interval) << 3) << "\n";
  cout << "(-infty) >> 3\t= " << (Minus_infinity (Interval) >> 3) << "\n";
  return 0;
}
