
/******************************************************************************
* MODULE     : relaxed_test.cc
* DESCRIPTION: Test relaxed power series
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

//#include "rational.hpp"
#include "numeric_extra.hpp"
#include "relaxed_series.hpp"

/******************************************************************************
* Test series
******************************************************************************/

namespace mml {

template<typename C, typename V>
class test_series_rep: public series_rep<C,V> {
public:
  inline test_series_rep () {}
  generic As_generic (const generic& z) const {
    return as_generic ("test", z); }
  C next () {
    return C (1); }
};

template<typename C, typename V> series<C,V>
test_series () {
  return (series_rep<C,V>*) new test_series_rep<C,V> ();
}

} // end namespace mml

/******************************************************************************
* Main tests
******************************************************************************/

using namespace mml;

//#define C rational
#define C complex_double
#define V relaxed<naive_multiply>
#define Series series<C,V >

int
main () {
  Series z (1, 1);
  Series t= test_series<C,V > ();
  Series f= 1 / (1 - z);
  Series g= exp (z);
  Series h= exp (shift_exp (t, 1));

  (void) set_series_precision (25);
  cout << "z\t= " << z << "\n\n";
  cout << "t\t= " << t << "\n\n";
  cout << "t*t\t= " << t*t << "\n\n";
  cout << "f\t= " << f << "\n\n";
  cout << "g\t= " << g << "\n\n";
  cout << "h\t= " << h << "\n\n";

  return 0;
}
