
/******************************************************************************
* MODULE     : skew_polynomial.hpp
* DESCRIPTION: Skew polynomials
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_SKEW_POLYNOMIAL_HPP
#define __MML_SKEW_POLYNOMIAL_HPP
#include "dense_polynomial.hpp"
#include "skew_jet.hpp"
#include "generic_symbols.hpp"
namespace mml {

/******************************************************************************
* The operator d/dx
******************************************************************************/

#define Skew_multiply skew_multiply<d_operator>

template<typename C> inline generic
var (const polynomial<C,dense_polynomial<Skew_multiply > >& P) {
  (void) P;
  return MML_PARTIAL;
}

template<typename C> polynomial<C,dense_polynomial<Skew_multiply > >
adjoint (const polynomial<C,dense_polynomial<Skew_multiply > >& P) {
  nat n= dim (P);
  if (n <= 1) return P;
  C* r= new C[n];
  jet_dicho_adjoint<C,d_operator> (r, seg (P), n, false);
  return polynomial<C,dense_polynomial<Skew_multiply > > (r, n);
}

#undef Skew_multiply

/******************************************************************************
* The operator x d/dx
******************************************************************************/

#define Skew_multiply skew_multiply<xd_operator>

template<typename C> generic
var (const polynomial<C,dense_polynomial<Skew_multiply > >& P) {
  (void) P;
  return MML_DELTA;
}

template<typename C> polynomial<C,dense_polynomial<Skew_multiply > >
adjoint (const polynomial<C,dense_polynomial<Skew_multiply > >& P) {
  nat n= dim (P);
  if (n <= 1) return P;
  C* r= new C[n];
  jet_dicho_adjoint<C,xd_operator> (r, seg (P), n, false);
  return polynomial<C,dense_polynomial<Skew_multiply > > (r, n);
}

#undef Skew_multiply

/******************************************************************************
* The shift operators x |-> x + 1 and x |-> x - 1
******************************************************************************/

#define Skew_multiply skew_multiply<S_operator<K> >
#define ISkew_multiply skew_multiply<T_operator<K> >

template<typename C, typename K> inline generic
var (const polynomial<C,dense_polynomial<
                          Skew_multiply > >& P) {
  (void) P;
  return "S";
}

template<typename C, typename K> inline generic
var (const polynomial<C,dense_polynomial<ISkew_multiply > >& P) {
  (void) P;
  return "T";
}

template<typename C, typename K>
polynomial<C,dense_polynomial<ISkew_multiply > >
adjoint (const polynomial<C,dense_polynomial<Skew_multiply > >& P) {
  nat n= dim (P);
  C* r= new C[n];
  if (n>1) jet_dicho_adjoint<C,S_operator<K> > (r, seg (P), n, false);
  return polynomial<C,dense_polynomial<ISkew_multiply > > (r, n);
}

template<typename C, typename K>
polynomial<C,dense_polynomial<Skew_multiply > >
adjoint (const polynomial<C,dense_polynomial<ISkew_multiply > >& P) {
  nat n= dim (P);
  C* r= new C[n];
  if (n>1) jet_dicho_adjoint<C,T_operator<K> > (r, seg (P), n, false);
  return polynomial<C,dense_polynomial<Skew_multiply > > (r, n);
}

#undef Skew_multiply
#undef ISkew_multiply

} // namespace mml
#endif // __MML_SKEW_POLYNOMIAL_HPP
