
/******************************************************************************
* MODULE     : generic.hpp
* DESCRIPTION: Generic expressions
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

// Notice that generic is included by base.hpp

#ifndef __GENERIC_HPP
#define __GENERIC_HPP
namespace mml {

class generic;
class string;

class generic_rep: public rep_struct {
protected:
  virtual nat get_type () const = 0;
  virtual nat get_dim () const = 0;
  virtual generic get_child (nat i) const = 0;
  virtual nat get_hash_value () const = 0;
  virtual bool identical (const generic& rep) const = 0;
  virtual generic flatten (bool recursive) const = 0;

public:
  inline generic_rep () {}
  inline virtual ~generic_rep () {}

  friend class generic;
  inline friend nat type (const generic& g);
  inline friend nat N (const generic& g);
  inline friend nat hash (const generic& g);
  inline friend generic flatten (const generic& s);
  inline friend generic flatten (const generic& g, bool really);
};

class generic {
INDIRECT_PROTO (generic, generic_rep)
public:
  generic ();
  generic (int i);
  generic (char* s);
  generic (const string& s);
  generic operator () () const;
  generic operator () (const generic& g1) const;
  generic operator () (const generic& g1, const generic& g2) const;
  generic operator () (const generic& g1, const generic& g2,
		       const generic& g3) const;

  inline friend generic_rep* inspect (const generic& g) { return g.rep; }
  inline friend nat type (const generic& g) { return g->get_type (); }
  inline friend nat N (const generic& g) { return g->get_dim (); }
  inline generic operator [] (nat i) const { return rep->get_child (i); }
  inline friend nat hash (const generic& g) { return g->get_hash_value (); }
  inline bool operator == (const generic& g) const {
    return rep->identical (g); }
  inline bool operator != (const generic& g) const {
    return !rep->identical (g); }
  inline friend generic flatten (const generic& s) {
    return s.rep->flatten (true); }
  inline friend generic flatten (const generic& g, bool really) {
    return really? g.rep->flatten (true): g; }

  friend class generic_rep;
};
INDIRECT_IMPL (generic, generic_rep)

void lisp_print (ostream& out, const generic& g);
void pretty_print (ostream& out, const generic& g);
ostream& operator << (ostream& out, const generic& g);
generic from_lisp (const string& s);

} // namespace mml
#endif // __GENERIC_HPP
