
/******************************************************************************
* MODULE     : generic_internal.hpp
* DESCRIPTION: Internal generic expression types
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __GENERIC_INTERNAL_HPP
#define __GENERIC_INTERNAL_HPP
#include "generic_symbols.hpp"
#include "symbol.hpp"
#include "string.hpp"
#include "dense_vector.hpp"
#include "table.hpp"
namespace mml {

#define GENERIC_SCALAR    0
#define GENERIC_ATOM      1
#define GENERIC_COMPOUND  2
#define GENERIC_SUM       3
#define GENERIC_PRODUCT   4
#define GENERIC_CODE      5

template<class C>
class generic_scalar_rep: public generic_rep {
public:
  C data;

protected:
  nat get_type () const { return GENERIC_SCALAR; }
  nat get_dim () const { return 0; }
  generic get_child (nat i) const {
    assert (false); // illegal access
    return generic (); }
  nat get_hash_value () const { return hash (data); }
  bool identical (const generic& g) const {
    if (type (g) != GENERIC_SCALAR) return false;
    return data == ((generic_scalar_rep<C>*) inspect (g)) -> data; }
  generic flatten (bool recursive) const {
    (void) recursive;
    return as_generic (data); }

public:
  generic_scalar_rep (const C& data2): data (data2) {}
};

inline bool is_scalar (const generic& g) {
  return type (g) == GENERIC_SCALAR; }
template<class C> generic generic_scalar (const C& sc) {
  return new generic_scalar_rep<C> (sc); }
template<class C> C as_scalar (const generic& g) {
  return ((generic_scalar_rep<C>*) inspect (g)) -> data; }

inline bool is_atom (const generic& g) {
  return type (g) == GENERIC_ATOM; }
generic generic_atom (char* s);
generic generic_atom (const string& s);
symbol<string> as_atom (const generic& g);
string as_string (const generic& g);

inline bool is_compound (const generic& g) {
  return type (g) == GENERIC_COMPOUND; }
generic generic_compound (const vector<generic>& v);
generic generic_compound (const generic& g1);
generic generic_compound (const generic& g1, const generic& g2);
generic generic_compound (const generic& g1, const generic& g2,
			  const generic& g3);
symbol<vector<generic> > as_compound (const generic& g);
vector<generic> as_vector (const generic& g);

inline bool is_sum (const generic& g) {
  return type (g) == GENERIC_SUM; }
generic generic_sum (const table<generic,generic>& t);
table<generic,generic> as_sum (const generic& g);

inline bool is_product (const generic& g) {
  return type (g) == GENERIC_PRODUCT; }
generic generic_product (const table<generic,generic>& t);
table<generic,generic> as_product (const generic& g);

inline bool is_func (const generic& g, const generic& f) {
  return is_compound (g) && g[0] == f; }
inline bool is_func (const generic& g, const generic& f, nat n) {
  return is_compound (g) && N (g) == (n+1) && g[0] == f; }
nat size (const generic& g);
int big_small_compare (const generic& g1, const generic& g2);
int small_big_compare (const generic& g1, const generic& g2);

} // namespace mml
#endif // __GENERIC_INTERNAL_HPP
