
/******************************************************************************
* MODULE     : generic_sum.cpp
* DESCRIPTION: Linear combinations of generics represented by hash tables
* COPYRIGHT  : (C) 2005  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "generic_internal.hpp"
#include "list_sort.hpp"
namespace mml {

class generic_sum_rep: public generic_rep {
public:
  symbol<table<generic,generic> > data;

protected:
  nat get_type () const { return GENERIC_SUM; }
  nat get_dim () const { return 0; }
  generic get_child (nat i) const {
    assert (false); // illegal access
    return generic (); }
  nat get_hash_value () const { return hash (data); }
  bool identical (const generic& g) const {
    if (type (g) != GENERIC_SUM) return false;
    return data == ((generic_sum_rep*) inspect (g)) -> data; }
  generic flatten (bool recursive) const {
    table<generic,generic> sum= *data;
    list<generic> l (entries (sum));
    l= sort (l, big_small_compare);
    generic acc= 0;
    while (!nil (l)) {
      generic mon  = mml::flatten (car (l), recursive);
      generic coeff= mml::flatten (sum[car (l)], recursive);
      acc= plus (acc, times (coeff, mon, true));
      l= cdr (l);
    }
    return migrate_negate (acc, 5);
  }

public:
  generic_sum_rep (const table<generic,generic>& data2): data (data2) {}
};

generic
generic_sum (const table<generic,generic>& t) {
  return new generic_sum_rep (t);
}

table<generic,generic>
as_sum (const generic& g) {
  return * ((generic_sum_rep*) inspect (g)) -> data;
}

} // namespace mml
